* {{
    font-family: 'Lato'; 
}}
body {{ 
    word-wrap: break-word;
}}
p {{ 
    color: #000;
}}
a {{
    color: #000;
}}
.msg-user {{
    color: #3a3a3a !important;
    white-space: pre-wrap;
    width: 100%;
    max-width: 100%;
}}
.msg-user p {{
    color: #3a3a3a !important;
}}
.msg-bot {{
    color: #000;
    white-space: pre-wrap;
    width: 100%;
    max-width: 100%;
}}
.cmd {{
    color: #4d4d4d;
    text-decoration: none;
}}
.ts {{
    color: #4d4d4d;
}}
.list {{
}}
pre {{
    color: #000;
    background-color: #e9e9e9;
    font-family: 'Monaspace Neon';
    display: block;
    word-wrap: break-word;
    white-space: pre-wrap;
}}
code {{
    color: #000;
    word-wrap: break-word;
    font-family: 'Monaspace Neon';
}}
.code-wrapper {{
    background-color: #e9e9e9;
}}
.code-header-lang {{
    color: #a0a0a0;
}}